"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = getPluginInjectNodeGlobals;

var _cloneDeep = _interopRequireDefault(require("lodash/cloneDeep"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/* eslint-disable no-param-reassign,prefer-destructuring */
const INJECTION_NAMES = new Set(['setImmediate', 'clearImmediate', 'Buffer', 'process']);

function makeTemplate(content) {
  let cachedValue;
  return function ({
    template
  }) {
    if (!cachedValue) {
      cachedValue = template.ast(content);
    }

    return (0, _cloneDeep.default)(cachedValue);
  };
}

const getTimersInjection = makeTemplate(`var __$sb$pundle$timers = require('timers'),
        setImmediate = __$sb$pundle$timers.setImmediate,
        clearImmediate=__$sb$pundle$timers.clearImmediate`);
const getBufferInjection = makeTemplate(`var Buffer = require('buffer').Buffer`);
const getProcessInjection = makeTemplate(`var process = require('process')`);
const INJECTIONS = Symbol('Node Global Injections');

function getPluginInjectNodeGlobals({
  template
}) {
  return {
    visitor: {
      Identifier(path) {
        const node = path.node;

        if (INJECTION_NAMES.has(node.name) && !path.hub[INJECTIONS].has(node.name) && path.isReferencedIdentifier() && !path.scope.hasBinding(node.name)) {
          path.hub[INJECTIONS].add(node.name);
        }
      },

      Program: {
        enter(path) {
          path.hub[INJECTIONS] = new Set();
        },

        exit(path) {
          const injections = path.hub[INJECTIONS];
          if (!injections.size) return;
          const statements = [];

          if (injections.has('setImmediate') || injections.has('clearImmediate')) {
            statements.push(getTimersInjection({
              template
            }));
          }

          if (injections.has('Buffer')) {
            statements.push(getBufferInjection({
              template
            }));
          }

          if (injections.has('process')) {
            statements.push(getProcessInjection({
              template
            }));
          }

          path.node.body = statements.concat(path.node.body);
        }

      }
    }
  };
}